==> mrpt_1.0.0.orig.tar.gz: mrpt-1.0.0/libs/maps/src/maps/CReflectivityGridMap2D.cpp <==
/* +---------------------------------------------------------------------------+
   |                 The Mobile Robot Programming Toolkit (MRPT)               |
   |                                                                           |
   |                          http://www.mrpt.org/                             |
   |                                                                           |
   | Copyright (c) 2005-2013, Individual contributors, see AUTHORS file        |
   | Copyright (c) 2005-2013, MAPIR group, University of Malaga                |
   | Copyright (c) 2012-2013, University of Almeria                            |
   | All rights reserved.                                                      |
   |                                                                           |
   | Redistribution and use in source and binary forms, with or without        |
   | modification, are permitted provided that the following conditions are    |
   | met:                                                                      |
   |    * Redistributions of source code must retain the above copyright       |
   |      notice, this list of conditions and the following disclaimer.        |
   |    * Redistributions in binary form must reproduce the above copyright    |
   |      notice, this list of conditions and the following disclaimer in the  |
   |      documentation and/or other materials provided with the distribution. |
   |    * Neither the name of the copyright holders nor the                    |
   |      names of its contributors may be used to endorse or promote products |
   |      derived from this software without specific prior written permission.|
   |                                                                           |
   | THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       |
   | 'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED |
   | TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR|
   | PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE |
   | FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL|
   | DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR|
   |  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       |
   | HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,       |
   | STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN  |
   | ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE           |
   | POSSIBILITY OF SUCH DAMAGE.                                               |
   +---------------------------------------------------------------------------+ */

#include <mrpt/maps.h>  // Precompiled header

#include <mrpt/slam/CReflectivityGridMap2D.h>
#include <mrpt/slam/CObservationReflectivity.h>

#include <mrpt/opengl/CTexturedPlane.h>


using namespace mrpt;
using namespace mrpt::slam;
using namespace mrpt::poses;
using namespace mrpt::math;
using namespace mrpt::utils;
using namespace mrpt::system;
using namespace std;

IMPLEMENTS_SERIALIZABLE(CReflectivityGridMap2D, CMetricMap,mrpt::slam)


// Lookup tables for log-odds
CLogOddsGridMapLUT<CReflectivityGridMap2D::cell_t>  CReflectivityGridMap2D::m_logodd_lut;

/*---------------------------------------------------------------
						Constructor
  ---------------------------------------------------------------*/
CReflectivityGridMap2D::CReflectivityGridMap2D(
	float		x_min,
	float		x_max,
	float		y_min,
	float		y_max,
	float		resolution ) :
		CDynamicGrid<int8_t>( x_min,x_max,y_min,y_max,resolution ),
		insertionOptions()
{
	internal_clear();
}

/*---------------------------------------------------------------
						clear
  ---------------------------------------------------------------*/
void  CReflectivityGridMap2D::internal_clear()
{
	fill( m_logodd_lut.p2l(0.5) );
}

/*---------------------------------------------------------------
						isEmpty
  ---------------------------------------------------------------*/
bool  CReflectivityGridMap2D::isEmpty() const
{
	return false;
}

/*---------------------------------------------------------------
						insertObservation
  ---------------------------------------------------------------*/
bool  CReflectivityGridMap2D::internal_insertObservation(
	const CObservation	*obs,
	const CPose3D			*robotPose )
{
	MRPT_START

	CPose2D		robotPose2D;
	CPose3D		robotPose3D;

	if (robotPose)
	{
		robotPose2D = CPose2D(*robotPose);
		robotPose3D = (*robotPose);
	}
	else
	{
		// Default values are (0,0,0)
	}

	if ( IS_CLASS(obs, CObservationReflectivity ) )
	{
		/********************************************************************
					OBSERVATION TYPE: CObservationReflectivity
		********************************************************************/
		const CObservationReflectivity	*o = static_cast<const CObservationReflectivity*>( obs );

		CPose3D sensor_pose;
		sensor_pose.composeFrom(robotPose3D, o->sensorPose);

		// log-odd increment due to the observation:
		const cell_t logodd_observation = m_logodd_lut.p2l( o->reflectivityLevel );

		// Update cell, with saturation:
		cell_t *cell = cellByPos(sensor_pose.x(),sensor_pose.y());
		if (!cell)
		{
			// We need to resize the grid!
			const float new_x_min = std::min( m_x_min, (float)sensor_pose.x());
			const float new_y_min = std::min( m_y_min, (float)sensor_pose.y());
			const float new_x_max = std::min( m_x_max, (float)sensor_pose.x());
			const float new_y_max = std::min( m_y_max, (float)sensor_pose.y());

			const int8_t default_cell = m_logodd_lut.p2l(0.5);
			resize(new_x_min,new_x_max, new_y_min,new_y_max, default_cell , 2.0 /* addit. margin */ );

			// Now we should get the cell:
			cell = cellByPos(sensor_pose.x(),sensor_pose.y());

			ASSERTMSG_(cell!=NULL,"cell==NULL even after resizing grid!?")
		}

		const int cell_old = static_cast<int>(*cell);
		int cell_new = cell_old + static_cast<int>(logodd_observation);
		keep_min(cell_new, static_cast<int>( CELLTYPE_MAX)-1);
		keep_max(cell_new, static_cast<int>( CELLTYPE_MIN)+1);

		*cell = static_cast<cell_t>(cell_new);

		return true;	// Done!
	} // end if "CObservationGasSensors"

	return false;

	MRPT_END
}


/*---------------------------------------------------------------
						computeObservationLikelihood
  ---------------------------------------------------------------*/
double	 CReflectivityGridMap2D::computeObservationLikelihood(
	const CObservation		*obs,
	const CPose3D			&takenFrom )
{
	MRPT_START

	if ( IS_CLASS(obs, CObservationReflectivity ) )
	{
		/********************************************************************
					OBSERVATION TYPE: CObservationReflectivity
		********************************************************************/
		const CObservationReflectivity	*o = static_cast<const CObservationReflectivity*>( obs );

		CPose3D sensor_pose;
		sensor_pose.composeFrom(takenFrom, o->sensorPose);

		cell_t *cell = cellByPos(sensor_pose.x(),sensor_pose.y());
		if (!cell)
			return 0; // out of the map..
		else
		{
			ASSERT_ABOVEEQ_(o->reflectivityLevel,0)
			ASSERT_BELOWEQ_(o->reflectivityLevel,1)
			return -0.5*square( ( m_logodd_lut.p2l(*cell) - o->reflectivityLevel )/o->sensorStdNoise);
		}
	}
	else
		return 0;

	MRPT_END
}

/*---------------------------------------------------------------
  Implements the writing to a CStream capability of CSerializable objects
 ---------------------------------------------------------------*/
void  CReflectivityGridMap2D::writeToStream(CStream &out, int *version) const
{
	if (version)
		*version = 0;
	else
	{
		// Save the dimensions of the grid:
		out << m_x_min << m_x_max << m_y_min << m_y_max;
		out << m_resolution;
		out << static_cast<uint32_t>(m_size_x) << static_cast<uint32_t>(m_size_y);

		// Map cells:
		const uint32_t n = static_cast<uint32_t>(m_map.size());
		out << n;
		if (n)
			out.WriteBuffer(&m_map[0],n);

		// Save the insertion options:
		// out << insertionOptions.maxOccupancyUpdateCertainty;

	}
}

/*---------------------------------------------------------------
  Implements the reading from a CStream capability of CSerializable objects
 ---------------------------------------------------------------*/
void  CReflectivityGridMap2D::readFromStream(CStream &in, int version)
{
	switch(version)
	{
	case 0:
		{
			uint32_t	n,i,j;

			// Load the dimensions of the grid:
			in >> m_x_min >> m_x_max >> m_y_min >> m_y_max;
			in >> m_resolution;
			in >> i >> j;
			m_size_x = i;
			m_size_y = j;

			// Map cells:
			in >> n;
			m_map.resize(n);
			if (n)
				in.ReadBuffer(&m_map[0],n);

			// Load the insertion options:
			// in >> insertionOptions.maxOccupancyUpdateCertainty;

		} break;
	default:
		MRPT_THROW_UNKNOWN_SERIALIZATION_VERSION(version)

	};

}

/*---------------------------------------------------------------
					TInsertionOptions
 ---------------------------------------------------------------*/
CReflectivityGridMap2D::TInsertionOptions::TInsertionOptions()
{
}

/*---------------------------------------------------------------
					dumpToTextStream
  ---------------------------------------------------------------*/
void  CReflectivityGridMap2D::TInsertionOptions::dumpToTextStream(CStream	&out) const
{
	out.printf("\n----------- [CReflectivityGridMap2D::TInsertionOptions] ------------ \n\n");

	//LOADABLEOPTS_DUMP_VAR(maxOccupancyUpdateCertainty, float)

	out.printf("\n");
}

/*---------------------------------------------------------------
					loadFromConfigFile
  ---------------------------------------------------------------*/
void  CReflectivityGridMap2D::TInsertionOptions::loadFromConfigFile(
	const mrpt::utils::CConfigFileBase  &iniFile,
	const std::string &section)
{
	//MRPT_LOAD_CONFIG_VAR( maxOccupancyUpdateCertainty,	float, iniFile, section )
}

/*---------------------------------------------------------------
					saveMetricMapRepresentationToFile
  ---------------------------------------------------------------*/
void  CReflectivityGridMap2D::saveMetricMapRepresentationToFile(
	const std::string	&filNamePrefix ) const
{
	std::string		fil;

	// Text matrix:
	saveToTextFile( filNamePrefix + std::string("_probability.txt") );
}

/*---------------------------------------------------------------
					getAsImage
  ---------------------------------------------------------------*/
void  CReflectivityGridMap2D::getAsImage(
	utils::CImage	&img,
	bool verticalFlip,
	bool forceRGB ) const
{
	if (!forceRGB)
	{	// 8bit gray-scale
		img.resize(m_size_x,m_size_y,1,true);
		const cell_t	*srcPtr = &m_map[0];
		unsigned char	*destPtr;
		for (unsigned int y=0;y<m_size_y;y++)
		{
			if (!verticalFlip)
					destPtr = img(0,m_size_y-1-y);
			else 	destPtr = img(0,y);
			for (unsigned int x=0;x<m_size_x;x++)
			{
				*destPtr++ = m_logodd_lut.l2p_255(*srcPtr++);
			}
		}
	}
	else
	{	// 24bit RGB:
		img.resize(m_size_x,m_size_y,3,true);
		const cell_t	*srcPtr = &m_map[0];
		unsigned char	*destPtr;
		for (unsigned int y=0;y<m_size_y;y++)
		{
			if (!verticalFlip)
					destPtr = img(0,m_size_y-1-y);
			else 	destPtr = img(0,y);
			for (unsigned int x=0;x<m_size_x;x++)
			{
				uint8_t c = m_logodd_lut.l2p_255(*srcPtr++);
				*destPtr++ = c;
				*destPtr++ = c;
				*destPtr++ = c;
			}
		}
	}
}


/*---------------------------------------------------------------
						getAs3DObject
---------------------------------------------------------------*/
void  CReflectivityGridMap2D::getAs3DObject( mrpt::opengl::CSetOfObjectsPtr	&outSetOfObj ) const
{
	if (m_disableSaveAs3DObject)
		return;

	MRPT_START

	opengl::CTexturedPlanePtr	outObj = opengl::CTexturedPlane::Create();

	outObj->setPlaneCorners(m_x_min,m_x_max,m_y_min,m_y_max);

	// Create the color & transparecy (alpha) images:
	CImage			imgColor(m_size_x,m_size_y,1);
	CImage			imgTrans(m_size_x,m_size_y,1);

	const cell_t		*srcPtr = &m_map[0];
	unsigned char		*destPtr_color;
	unsigned char		*destPtr_trans;

	for (unsigned int y=0;y<m_size_y;y++)
	{
		destPtr_color = imgColor(0,y);
		destPtr_trans = imgTrans(0,y);
		for (unsigned int x=0;x<m_size_x;x++)
		{
			uint8_t  cell255 = m_logodd_lut.l2p_255(*srcPtr++);
			*destPtr_color++ = cell255;

			int8_t   auxC = (int8_t)((signed short)cell255)-128;
			*destPtr_trans++ = auxC>0 ? (auxC << 1) : ((-auxC) << 1);
		}
	}

	outObj->assignImage_fast( imgColor,imgTrans );
	outSetOfObj->insert( outObj );

	MRPT_END
}

/*---------------------------------------------------------------
  Computes the ratio in [0,1] of correspondences between "this" and the "otherMap" map, whose 6D pose relative to "this" is "otherMapPose"
 *   In the case of a multi-metric map, this returns the average between the maps. This method always return 0 for grid maps.
 * \param  otherMap					  [IN] The other map to compute the matching with.
 * \param  otherMapPose				  [IN] The 6D pose of the other map as seen from "this".
 * \param  minDistForCorr			  [IN] The minimum distance between 2 non-probabilistic map elements for counting them as a correspondence.
 * \param  minMahaDistForCorr		  [IN] The minimum Mahalanobis distance between 2 probabilistic map elements for counting them as a correspondence.
 *
 * \return The matching ratio [0,1]
 * \sa computeMatchingWith2D
  --------------------------------------------------------------- */
float  CReflectivityGridMap2D::compute3DMatchingRatio(
		const CMetricMap						*otherMap,
		const CPose3D							&otherMapPose,
		float									minDistForCorr,
		float									minMahaDistForCorr
		) const
{
	MRPT_UNUSED_PARAM(otherMap);
	MRPT_UNUSED_PARAM(otherMapPose);
	MRPT_UNUSED_PARAM(minDistForCorr);
	MRPT_UNUSED_PARAM(minMahaDistForCorr);

	return 0;
}

==> mrpt_1.2.1.orig.tar.gz: mrpt-1.2.1/libs/maps/src/maps/CReflectivityGridMap2D.cpp <==
/* +---------------------------------------------------------------------------+
   |                     Mobile Robot Programming Toolkit (MRPT)               |
   |                          http://www.mrpt.org/                             |
   |                                                                           |
   | Copyright (c) 2005-2014, Individual contributors, see AUTHORS file        |
   | See: http://www.mrpt.org/Authors - All rights reserved.                   |
   | Released under BSD License. See details in http://www.mrpt.org/License    |
   +---------------------------------------------------------------------------+ */

#include "maps-precomp.h" // Precomp header

#include <mrpt/slam/CReflectivityGridMap2D.h>
#include <mrpt/slam/CObservationReflectivity.h>
#include <mrpt/poses/CPose2D.h>
#include <mrpt/utils/round.h> // round()
#include <mrpt/system/os.h>
#include <mrpt/opengl/CTexturedPlane.h>
#include <mrpt/opengl/CSetOfObjects.h>
#include <mrpt/utils/CStream.h>


using namespace mrpt;
using namespace mrpt::slam;
using namespace mrpt::poses;
using namespace mrpt::math;
using namespace mrpt::utils;
using namespace mrpt::system;
using namespace std;

IMPLEMENTS_SERIALIZABLE(CReflectivityGridMap2D, CMetricMap,mrpt::slam)


// Lookup tables for log-odds
CLogOddsGridMapLUT<CReflectivityGridMap2D::cell_t>  CReflectivityGridMap2D::m_logodd_lut;

/*---------------------------------------------------------------
						Constructor
  ---------------------------------------------------------------*/
CReflectivityGridMap2D::CReflectivityGridMap2D(
	float		x_min,
	float		x_max,
	float		y_min,
	float		y_max,
	float		resolution ) :
		CDynamicGrid<int8_t>( x_min,x_max,y_min,y_max,resolution ),
		insertionOptions()
{
	internal_clear();
}

/*---------------------------------------------------------------
						clear
  ---------------------------------------------------------------*/
void  CReflectivityGridMap2D::internal_clear()
{
	fill( m_logodd_lut.p2l(0.5) );
}

/*---------------------------------------------------------------
						isEmpty
  ---------------------------------------------------------------*/
bool  CReflectivityGridMap2D::isEmpty() const
{
	return false;
}

/*---------------------------------------------------------------
						insertObservation
  ---------------------------------------------------------------*/
bool  CReflectivityGridMap2D::internal_insertObservation(
	const CObservation	*obs,
	const CPose3D			*robotPose )
{
	MRPT_START

	CPose2D		robotPose2D;
	CPose3D		robotPose3D;

	if (robotPose)
	{
		robotPose2D = CPose2D(*robotPose);
		robotPose3D = (*robotPose);
	}
	else
	{
		// Default values are (0,0,0)
	}

	if ( IS_CLASS(obs, CObservationReflectivity ) )
	{
		/********************************************************************
					OBSERVATION TYPE: CObservationReflectivity
		********************************************************************/
		const CObservationReflectivity	*o = static_cast<const CObservationReflectivity*>( obs );

		CPose3D sensor_pose;
		sensor_pose.composeFrom(robotPose3D, o->sensorPose);

		// log-odd increment due to the observation:
		const cell_t logodd_observation = m_logodd_lut.p2l( o->reflectivityLevel );

		// Update cell, with saturation:
		cell_t *cell = cellByPos(sensor_pose.x(),sensor_pose.y());
		if (!cell)
		{
			// We need to resize the grid!
			const float new_x_min = std::min( m_x_min, (float)sensor_pose.x());
			const float new_y_min = std::min( m_y_min, (float)sensor_pose.y());
			const float new_x_max = std::min( m_x_max, (float)sensor_pose.x());
			const float new_y_max = std::min( m_y_max, (float)sensor_pose.y());

			const int8_t default_cell = m_logodd_lut.p2l(0.5);
			resize(new_x_min,new_x_max, new_y_min,new_y_max, default_cell , 2.0 /* addit. margin */ );

			// Now we should get the cell:
			cell = cellByPos(sensor_pose.x(),sensor_pose.y());

			ASSERTMSG_(cell!=NULL,"cell==NULL even after resizing grid!?")
		}

		const int cell_old = static_cast<int>(*cell);
		int cell_new = cell_old + static_cast<int>(logodd_observation);
		keep_min(cell_new, static_cast<int>( CELLTYPE_MAX)-1);
		keep_max(cell_new, static_cast<int>( CELLTYPE_MIN)+1);

		*cell = static_cast<cell_t>(cell_new);

		return true;	// Done!
	} // end if "CObservationGasSensors"

	return false;

	MRPT_END
}


/*---------------------------------------------------------------
						computeObservationLikelihood
  ---------------------------------------------------------------*/
double	 CReflectivityGridMap2D::computeObservationLikelihood(
	const CObservation		*obs,
	const CPose3D			&takenFrom )
{
	MRPT_START

	if ( IS_CLASS(obs, CObservationReflectivity ) )
	{
		/********************************************************************
					OBSERVATION TYPE: CObservationReflectivity
		********************************************************************/
		const CObservationReflectivity	*o = static_cast<const CObservationReflectivity*>( obs );

		CPose3D sensor_pose;
		sensor_pose.composeFrom(takenFrom, o->sensorPose);

		cell_t *cell = cellByPos(sensor_pose.x(),sensor_pose.y());
		if (!cell)
			return 0; // out of the map..
		else
		{
			ASSERT_ABOVEEQ_(o->reflectivityLevel,0)
			ASSERT_BELOWEQ_(o->reflectivityLevel,1)
			return -0.5*square( ( m_logodd_lut.p2l(*cell) - o->reflectivityLevel )/o->sensorStdNoise);
		}
	}
	else
		return 0;

	MRPT_END
}

/*---------------------------------------------------------------
  Implements the writing to a CStream capability of CSerializable objects
 ---------------------------------------------------------------*/
void  CReflectivityGridMap2D::writeToStream(CStream &out, int *version) const
{
	if (version)
		*version = 0;
	else
	{
		// Save the dimensions of the grid:
		out << m_x_min << m_x_max << m_y_min << m_y_max;
		out << m_resolution;
		out << static_cast<uint32_t>(m_size_x) << static_cast<uint32_t>(m_size_y);

		// Map cells:
		const uint32_t n = static_cast<uint32_t>(m_map.size());
		out << n;
		if (n)
			out.WriteBuffer(&m_map[0],n);

		// Save the insertion options:
		// out << insertionOptions.maxOccupancyUpdateCertainty;

	}
}

/*---------------------------------------------------------------
  Implements the reading from a CStream capability of CSerializable objects
 ---------------------------------------------------------------*/
void  CReflectivityGridMap2D::readFromStream(CStream &in, int version)
{
	switch(version)
	{
	case 0:
		{
			uint32_t	n,i,j;

			// Load the dimensions of the grid:
			in >> m_x_min >> m_x_max >> m_y_min >> m_y_max;
			in >> m_resolution;
			in >> i >> j;
			m_size_x = i;
			m_size_y = j;

			// Map cells:
			in >> n;
			m_map.resize(n);
			if (n)
				in.ReadBuffer(&m_map[0],n);

			// Load the insertion options:
			// in >> insertionOptions.maxOccupancyUpdateCertainty;

		} break;
	default:
		MRPT_THROW_UNKNOWN_SERIALIZATION_VERSION(version)

	};

}

/*---------------------------------------------------------------
					TInsertionOptions
 ---------------------------------------------------------------*/
CReflectivityGridMap2D::TInsertionOptions::TInsertionOptions()
{
}

/*---------------------------------------------------------------
					dumpToTextStream
  ---------------------------------------------------------------*/
void  CReflectivityGridMap2D::TInsertionOptions::dumpToTextStream(CStream	&out) const
{
	out.printf("\n----------- [CReflectivityGridMap2D::TInsertionOptions] ------------ \n\n");

	//LOADABLEOPTS_DUMP_VAR(maxOccupancyUpdateCertainty, float)

	out.printf("\n");
}

/*---------------------------------------------------------------
					loadFromConfigFile
  ---------------------------------------------------------------*/
void  CReflectivityGridMap2D::TInsertionOptions::loadFromConfigFile(
	const mrpt::utils::CConfigFileBase  &iniFile,
	const std::string &section)
{
	//MRPT_LOAD_CONFIG_VAR( maxOccupancyUpdateCertainty,	float, iniFile, section )
}

/*---------------------------------------------------------------
					saveMetricMapRepresentationToFile
  ---------------------------------------------------------------*/
void  CReflectivityGridMap2D::saveMetricMapRepresentationToFile(
	const std::string	&filNamePrefix ) const
{
	std::string		fil;

	// Text matrix:
	saveToTextFile( filNamePrefix + std::string("_probability.txt") );
}

/*---------------------------------------------------------------
					getAsImage
  ---------------------------------------------------------------*/
void  CReflectivityGridMap2D::getAsImage(
	utils::CImage	&img,
	bool verticalFlip,
	bool forceRGB ) const
{
	if (!forceRGB)
	{	// 8bit gray-scale
		img.resize(m_size_x,m_size_y,1,true);
		const cell_t	*srcPtr = &m_map[0];
		unsigned char	*destPtr;
		for (unsigned int y=0;y<m_size_y;y++)
		{
			if (!verticalFlip)
					destPtr = img(0,m_size_y-1-y);
			else 	destPtr = img(0,y);
			for (unsigned int x=0;x<m_size_x;x++)
			{
				*destPtr++ = m_logodd_lut.l2p_255(*srcPtr++);
			}
		}
	}
	else
	{	// 24bit RGB:
		img.resize(m_size_x,m_size_y,3,true);
		const cell_t	*srcPtr = &m_map[0];
		unsigned char	*destPtr;
		for (unsigned int y=0;y<m_size_y;y++)
		{
			if (!verticalFlip)
					destPtr = img(0,m_size_y-1-y);
			else 	destPtr = img(0,y);
			for (unsigned int x=0;x<m_size_x;x++)
			{
				uint8_t c = m_logodd_lut.l2p_255(*srcPtr++);
				*destPtr++ = c;
				*destPtr++ = c;
				*destPtr++ = c;
			}
		}
	}
}


/*---------------------------------------------------------------
						getAs3DObject
---------------------------------------------------------------*/
void  CReflectivityGridMap2D::getAs3DObject( mrpt::opengl::CSetOfObjectsPtr	&outSetOfObj ) const
{
	if (m_disableSaveAs3DObject)
		return;

	MRPT_START

	opengl::CTexturedPlanePtr	outObj = opengl::CTexturedPlane::Create();

	outObj->setPlaneCorners(m_x_min,m_x_max,m_y_min,m_y_max);

	// Create the color & transparecy (alpha) images:
	CImage			imgColor(m_size_x,m_size_y,1);
	CImage			imgTrans(m_size_x,m_size_y,1);

	const cell_t		*srcPtr = &m_map[0];
	unsigned char		*destPtr_color;
	unsigned char		*destPtr_trans;

	for (unsigned int y=0;y<m_size_y;y++)
	{
		destPtr_color = imgColor(0,y);
		destPtr_trans = imgTrans(0,y);
		for (unsigned int x=0;x<m_size_x;x++)
		{
			uint8_t  cell255 = m_logodd_lut.l2p_255(*srcPtr++);
			*destPtr_color++ = cell255;

			int8_t   auxC = (int8_t)((signed short)cell255)-128;
			*destPtr_trans++ = auxC>0 ? (auxC << 1) : ((-auxC) << 1);
		}
	}

	outObj->assignImage_fast( imgColor,imgTrans );
	outSetOfObj->insert( outObj );

	MRPT_END
}
